/*
/*

Copyright (C) 2006 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

 */

package com.clustercontrol.jobmanagement.view.action;

import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;

import com.clustercontrol.jobmanagement.util.JobEditStateUtil;
import com.clustercontrol.jobmanagement.util.JobEndpointWrapper;
import com.clustercontrol.jobmanagement.view.JobListView;
import com.clustercontrol.util.Messages;
import com.clustercontrol.ws.jobmanagement.InvalidRole_Exception;
import com.clustercontrol.ws.jobmanagement.JobInfo;

/**
 * ジョブ[一覧]ビューの「キャンセル」を行うクライアント側アクションクラス<BR>
 * 
 * @version 1.0.0
 * @since 1.0.0
 */
public class CancelJobAction implements IViewActionDelegate {

	// ログ
	private static Log m_log = LogFactory.getLog( CancelJobAction.class );

	/** ビュー */
	private IViewPart m_view;

	/**
	 * ビューを保持します。ワークベンチにロードされた際に呼ばれます。
	 * 
	 * @param view ビューのインスタンス
	 * 
	 * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
	 */
	@Override
	public void init(IViewPart view) {
		m_view = view;
	}

	/**
	 * ジョブ[一覧]ビューの「キャンセル」が押された場合に、ジョブ[一覧]ビューを更新します。
	 * <p>
	 * <ol>
	 * <li>確認ダイアログを表示します。</li>
	 * <li>ジョブ[一覧]ビューを更新します。</li>
	 * </ol>
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 * @see com.clustercontrol.jobmanagement.view.JobListView#update()
	 */
	@Override
	public void run(IAction action) {
		if (m_view instanceof JobListView) {
			JobListView view = (JobListView)m_view.getAdapter(JobListView.class);

			// 確認ダイアログを生成
			String message;
			if(JobEditStateUtil.isEdit()){
				message = Messages.getString("message.job.43") + "\n" +
						Messages.getString("message.job.30");
			}
			else{
				message = Messages.getString("message.job.30");
			}
			if (MessageDialog.openQuestion(
					null,
					Messages.getString("confirmed"),
					message)) {
				m_log.debug("CancelJob start " + new Date());

				// 編集モードから抜ける
				try {
					for (JobInfo info : JobEditStateUtil.getLockedJobunitList()) {
						JobEndpointWrapper.releaseEditLock(JobEditStateUtil.getEditSession(info));
					}
				} catch (InvalidRole_Exception e) {
					// アクセス権なしの場合、エラーダイアログを表示する
					MessageDialog.openInformation(
							null,
							Messages.getString("message"),
							Messages.getString("message.accesscontrol.16"));
				} catch (Exception e) {
					m_log.warn("updateJobunitUpdateTime() : " + e.getMessage(), e);
					MessageDialog.openError(
							null,
							Messages.getString("failed"),
							Messages.getString("message.hinemos.failure.unexpected") + ", " + e.getMessage());
				} finally {
					// ロックの開放の成功/失敗にかかわらず編集モードから抜ける
					JobEditStateUtil.clearEditStateAll();
				}

				view.update();
				view.getJobTreeComposite().refresh();
				m_log.debug("CancelJob end " + new Date());
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IAction action, ISelection selection) {

	}
}